<?php

namespace FiloBlu\AdminSecurity\Observer;

use FiloBlu\AdminSecurity\Helper\Data;
use FiloBlu\AdminSecurity\Model\Authentication;
use FiloBlu\AdminSecurity\Model\Ip;
use FiloBlu\AdminSecurity\Model\TrustedManager;
use Magento\Backend\Model\Auth\Session;
use Magento\Framework\Event\ObserverInterface;
use Magento\User\Model\UserFactory;

class PrepareLoginBackendObserver implements ObserverInterface
{
    /**
     * @var Session
     */
    protected $authStorage;

    /**
     * @var Ip
     */
    protected $ipModel;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var Authentication
     */
    protected $authModel;

    /**
     * @var TrustedManager
     */
    protected $trustedManager;

    /**
     * @var UserFactory
     */
    protected $userFactory;

    /**
     * @param Session $authStorage
     * @param Ip $ipModel
     * @param Data $helper
     * @param Authentication $authModel
     * @param UserFactory $userFactory
     */
    public function __construct(
        Session        $authStorage,
        Ip             $ipModel,
        Data           $helper,
        Authentication $authModel,
        TrustedManager $trustedManager,
        UserFactory    $userFactory
    )
    {
        $this->authStorage = $authStorage;
        $this->ipModel = $ipModel;
        $this->helper = $helper;
        $this->authModel = $authModel;
        $this->trustedManager = $trustedManager;
        $this->userFactory = $userFactory;
    }

    /**
     * Prepare Login Backend
     *
     * @param \Magento\Framework\Event\Observer $observer
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        $allowLoginCheck = false;
        if ($this->helper->adminSecurityIsEnabled()) {
            $allowLoginCheck = true;

            if ($this->ipModel->isIpInWhiteList()) {
                $allowLoginCheck = false;
            }

            $user = $this->userFactory->create()->loadByUsername($observer->getEvent()->getUsername());
            if ($this->trustedManager->isTrustedDevice($user)) {
                $allowLoginCheck = false;
                $this->trustedManager->rotateTrustedDeviceToken($user);
            }
        }

        if ($allowLoginCheck && is_null($this->authStorage->getUserSecData())) {
            $this->authStorage->setUserSecData($this->authModel->createAccountSecret());
        }

        $this->authStorage->setAllowBackendAccountLoginCheck($allowLoginCheck);
    }
}
